"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defineRoute = void 0;

var _queryString = _interopRequireDefault(require("query-string"));

var _react = _interopRequireDefault(require("react"));

var _reactRouterDom = require("react-router-dom");

var _resolvePath2 = require("./utils/resolvePath");

var _excluded = ["searchParams", "pathParams"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * Return typed path params
 */
var defineUseRouteParams = function defineUseRouteParams() {
  return function () {
    return (0, _reactRouterDom.useParams)();
  };
};
/**
 * Return default useLocation from react-router-dom
 */


var defineUseRouteLocation = function defineUseRouteLocation() {
  return function () {
    return (0, _reactRouterDom.useLocation)();
  };
};
/**
 * Return typed and parsed search params URL
 */


var defineUseRouteSearch = function defineUseRouteSearch() {
  return function () {
    var _useLocation = (0, _reactRouterDom.useLocation)(),
        search = _useLocation.search;

    return _queryString["default"].parse(search);
  };
};
/**
 * Wrapper on top of Route, prevent passing the path manually
 */


function withRoutePath(_ref) {
  var path = _ref.path;
  return function (props) {
    return /*#__PURE__*/_react["default"].createElement(_reactRouterDom.Route, _extends({
      path: path
    }, props), props.children);
  };
}
/**
 * Wrapper on top of Link, handle typesafe link with searchParams and pathParams
 */


function withLinkPath(_ref2) {
  var path = _ref2.path;
  return function (props) {
    var searchParams = props.searchParams,
        pathParams = props.pathParams,
        restProps = _objectWithoutProperties(props, _excluded);

    var resolvedPath = (0, _resolvePath2.resolvePath)({
      path: path,
      pathParams: pathParams,
      searchParams: searchParams
    });
    return /*#__PURE__*/_react["default"].createElement(_reactRouterDom.Link, _extends({
      to: resolvedPath
    }, restProps), props.children);
  };
}

var defineRoute = function defineRoute(routeOptions) {
  var currentPath = routeOptions.path;
  return {
    path: routeOptions.path,
    resolvePath: function resolvePath(_ref3) {
      var searchParams = _ref3.searchParams,
          pathParams = _ref3.pathParams;
      return (0, _resolvePath2.resolvePath)({
        searchParams: searchParams,
        pathParams: pathParams,
        path: routeOptions.path
      });
    },
    useParams: defineUseRouteParams(),
    useLocation: defineUseRouteLocation(),
    useSearch: defineUseRouteSearch(),
    Route: withRoutePath({
      path: currentPath
    }),
    Link: withLinkPath({
      path: currentPath
    })
  };
};

exports.defineRoute = defineRoute;